// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#include <stdlib.h>
#include <string.h>

#include "resize_fine.h"
#include "log.h"
#include "utils.h"

resize_fine::resize_fine()
{
	log(LL_INFO, "resize_fine instantiated");
}

resize_fine::~resize_fine()
{
}

typedef struct {
	int n;
	double r, g, b;
} pixel_t;

void resize_fine::do_resize_fine(const int win, const int hin, const uint8_t *const in, const int wout, const int hout, uint8_t **out)
{
	*out = (uint8_t *)allocate_aligned(wout * hout * 3);

	pixel_t *work = new pixel_t[wout * hout]();

	const double maxw = std::max(win, wout);
	const double maxh = std::max(hin, hout);

	const double hins = hin / maxh;
	const double wins = win / maxw;
	const double houts = hout / maxh;
	const double wouts = wout / maxw;

	for(int y=0; y<maxh; y++) {
		const int in_scaled_y = y * hins;
		const int in_scaled_o = in_scaled_y * win * 3;
		const int out_scaled_y = y * houts;
		const int out_scaled_o = out_scaled_y * wout * 3;

		for(int x=0; x<maxw; x++) {
			int ino = in_scaled_o + int(x * wins) * 3;
			int outo = out_scaled_o + int(x * wouts) * 3;

			work[outo].n++;
			work[outo].r += in[ino + 0];
			work[outo].g += in[ino + 1];
			work[outo].b += in[ino + 2];
		}
	}

	for(int y=0; y<hout; y++) {
		for(int x=0; x<wout; x++) {
			const int o = y * wout * 3 + x * 3;

			const int n = std::max(work[o].n, 1);
			(*out)[o + 0] = work[o].r / n;
			(*out)[o + 1] = work[o].g / n;
			(*out)[o + 2] = work[o].b / n;
		}
	}

	delete [] work;
}
